#include "game.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

// Help functions //
// -------------- //

void abort(void) {
    fprintf(stderr, "error: problem when loading board\n");
    exit(1);
}

// Functions //
// --------- //

void print_cell(enum cell cell) {
    char c;
    switch (cell) {
        case EMPTY:  c = '.'; break;
        case YELLOW: c = 'Y'; break;
        case RED:    c = 'R';
    };
    printf("%c", c);
}

void initialize_board(struct board *board) {
    for (unsigned int i = 0; i < 6; ++i)
        for (unsigned int j = 0; j < 7; ++j)
            board->cells[i][j] = EMPTY;
    board->player = YELLOW;
}

enum cell char2cell (char c) {
  switch (c) {
    case 'R' : return RED; break;
    case 'Y' : return YELLOW; break;
    default : return EMPTY; break;
  }
}

void load_board(struct board *board) {
    unsigned int n = 0;
    unsigned int i = 0, j = 0;
    char c;
    unsigned int reds = 0, yellows = 0;
    while (n < 42 && (c = getchar()) != EOF) {
        if (c == '.' || c == 'R' || c == 'Y') {
            enum cell cellule = char2cell(c);
            board->cells[i][j] = cellule;
//                c == '.' ? EMPTY : (c == 'R' ? RED : YELLOW);
            if (cellule == YELLOW) ++yellows;
            if (cellule == RED) ++reds;
            ++n;
            if (j == 6) {
                ++i;
                j = 0;
            } else {
                ++j;
            }
        }
    }
    if (n < 42) abort();
    board -> player = yellows > reds ? RED : YELLOW;
}

void print_board(const struct board *board,
                 const char *prefix) {
    for (unsigned int i = 0; i < 6; ++i) {
        printf("%s", prefix);
        for (unsigned int j = 0; j < 7; ++j) {
            print_cell(board->cells[i][j]);
        }
        printf("\n");
    }
    printf("%s%s to play\n", prefix,
           board->player == YELLOW ? "Yellow" : "Red");
}

bool can_play(const struct board *board,
              unsigned int j) {
    return j < 7 && board->cells[0][j] == EMPTY;
}

void switch_player(struct board *board) {
    board->player = 3 - board->player;
}

bool play(struct board *board,
          unsigned int j) {
    if (can_play(board, j)) {
        unsigned int i = 5;
        while (board->cells[i][j] != EMPTY) --i;
        board->cells[i][j] = board->player;
        switch_player(board);
        return true;
    } else {
        return false;
    }
}
