#ifndef CONNECT4_H
#define CONNECT4_H

#include <stdbool.h>

// Types //
// ----- //

enum cell {
    EMPTY  = 0, // Empty cell
    YELLOW = 1, // Yellow token
    RED    = 2, // Red token
};

struct board {
    enum cell player;      // The current player
    enum cell cells[6][7]; // The cells
};

// Functions //
// --------- //

/**
 * Print a cell to stdout
 *
 * @param cell  The cell
 */
void print_cell(enum cell cell);

/**
 * Initialize a board
 *
 * @param board  The board
 */
void initialize_board(struct board *board);

/**
 * Load board from stdin
 *
 * Validation is minimal: The first 42 valid characters are loaded row by row
 * and there is only an error if there are not enough character.
 *
 * @param  The board
 */
void load_board(struct board *board);

/**
 * Print a board to stdout
 *
 * @param board   The board
 * @param prefix  The prefix to add for each line
 */
void print_board(const struct board *board,
                 const char *prefix);

/**
 * Indicate if one can play in a board at the given column
 *
 * @param board  The board
 * @param j      The column
 * @return       True if the columns can be played
 */
bool can_play(const struct board *board,
              unsigned int j);

/**
 * Switch the current player
 *
 * @param board  The board
 */
void switch_player(struct board *board);

/**
 * Play at the given column
 *
 * @param board  The board
 * @param j      The column
 * @return       True if the play is allowed
 */
bool play(struct board *board,
          unsigned int j);

#endif
